@@ -1,12 +1,13 @@
 /* arp.h - ARP state machine */
  
-/* Written 1995 by Werner Almesberger, EPFL-LRC */
+/* Written 1995,1996 by Werner Almesberger, EPFL-LRC */
  
 
 #include <stdlib.h>
 #include <unistd.h>
 #include <string.h>
 #include <errno.h>
+#include <assert.h>
 #include <sys/types.h>
 #include <sys/socket.h> /* for linux/if_arp.h */
 #include <netinet/in.h> /* for ntohs, etc. */
@@ -28,12 +29,15 @@
 #define COMPONENT "ARP"
 
 
-#define T_REPLY	 1000000  /* 1 sec */
-#define R_REPLY	       5  /* 5 retries */
-#define T_REVAL	30000000  /* 30 sec - make much larger (if srv ...) @@@ */
-#define R_REVAL	       0  /* no retries */
-#define T_RETRY	 1000000  /* 1 sec */
-#define R_RETRY	      18  /* enough try keep trying during a long weekend */
+#define SEC	 1000000
+#define T_REPLY	    (1*SEC)	/* 1 sec */
+#define R_REPLY	    5		/* 5 retries */
+#define T_REVAL	    (30*SEC)	/* 30 sec - make much larger (if srv ...) @@@ */
+#define R_REVAL	    0		/* no retries */
+#define T_RETRY	    (1*SEC)	/* 1 sec */
+#define R_RETRY	    25000	/* enough try keep trying during a long
+				   weekend */
+#define MAX_DELAY   (300*SEC)	/* never wait more than 5 min for anything */
 
 
 #define ENTER_TIMER_HANDLER(entry) entry->timer = NULL
@@ -67,8 +71,11 @@ static void discard_entry(ENTRY *entry)
 	next = vcc->next;
 	discard_vcc(vcc);
     }
-    if (entry == entry->itf->arp_srv) entry->itf->arp_srv = NULL;
-    Q_REMOVE(entry->itf->table,entry);
+    if (!entry->itf) Q_REMOVE(unknown_incoming,entry);
+    else {
+	if (entry == entry->itf->arp_srv) entry->itf->arp_srv = NULL;
+	Q_REMOVE(entry->itf->table,entry);
+    }
     if (entry->addr) free(entry->addr);
     free(entry);
 }
@@ -91,9 +98,10 @@ static int want_arp_srv(ITF *itf)
     for (vcc = itf->arp_srv->vccs; vcc; vcc = vcc->next)
 	if (!vcc->connecting) return 1;
     if (itf->arp_srv->vccs) return 0;
-    if ((fd = connect_vcc((struct sockaddr *) itf->arp_srv->addr)) < 0)
-	return 0;
+    if ((fd = connect_vcc((struct sockaddr *) itf->arp_srv->addr,
+      CLIP_DEFAULT_IDLETIMER)) < 0) return 0;
     vcc = alloc_t(VCC);
+    vcc->active = 1;
     vcc->connecting = 1;
     vcc->fd = fd;
     vcc->entry = itf->arp_srv;
@@ -269,7 +277,7 @@ static void timer_expiration(void *user);
 static void revalidate(ENTRY *entry)
 {
     entry->state = as_resolv;
-    if (!want_arp_srv(entry->itf)) return;
+    if (want_arp_srv(entry->itf) <= 0) return;
     arp_request(entry->itf,entry->ip);
     START_TIMER(entry,REPLY);
 }
@@ -293,10 +301,11 @@ static void timer(ENTRY *entry)
 static void timeout(ENTRY *entry)
 {
     VCC *vcc;
+
     entry->timer = NULL;
     switch (entry->state) {
 	case as_resolv:
-	    if (entry->flags & ATF_ARPSRV && !entry->vccs) {
+	    if ((entry->flags & ATF_ARPSRV) && !entry->vccs) {
 		if (entry->itf) want_arp_srv(entry->itf);
 		break;
 	    }
@@ -311,8 +320,10 @@ static void timeout(ENTRY *entry)
 		break;
 	    }
 	    for (vcc = entry->vccs; vcc; vcc = vcc->next)
-		if (set_ip(vcc->fd,entry->ip) < 0)
-		    diag(COMPONENT,DIAG_ERROR,"set_ip(0): %s",strerror(errno));
+		if (!vcc->connecting)
+		    if (set_ip(vcc->fd,0) < 0)
+			diag(COMPONENT,DIAG_ERROR,"set_ip(0): %s",
+			  strerror(errno));
 	    if (entry->svc && entry->itf->arp_srv) revalidate(entry);
 	    else {
 		inarp_request(entry);
@@ -351,7 +362,8 @@ static void timer_expiration(void *user)
     else {
 	entry->retries--;
 	entry->timeout *= 2;
-	start_timer(entry->timeout,timer_expiration,entry);
+	if (entry->timeout > MAX_DELAY) entry->timeout = MAX_DELAY;
+	entry->timer = start_timer(entry->timeout,timer_expiration,entry);
 	timer(entry);
     }
 }
@@ -362,8 +374,10 @@ static void connect_me(ENTRY *entry)
     VCC *vcc;
     int fd;
 
-    if ((fd = connect_vcc((struct sockaddr *) entry->addr)) < 0) return;
+    if ((fd = connect_vcc((struct sockaddr *) entry->addr,
+      CLIP_DEFAULT_IDLETIMER)) < 0) return;
     vcc = alloc_t(VCC);
+    vcc->active = 1;
     vcc->connecting = 1;
     vcc->fd = fd;
     vcc->entry = entry;
@@ -387,12 +401,29 @@ static void learn(VCC *vcc,unsigned long ip,struct sockaddr_atmsvc *addr)
 	return;
     }
     entry = lookup_ip(itf,ip);
-    if (entry && vcc && (!vcc->entry || entry != vcc->entry)) {
+    assert(vcc->entry);
+    if (entry && vcc && vcc->entry->itf && entry != vcc->entry) {
 	diag(COMPONENT,DIAG_DEBUG,"collision on %d.%d.%d.%d",ipp[0],ipp[1],
 	  ipp[2],ipp[3]);
 	return;
     }
-    if (!entry && !(entry = vcc->entry)) {
+    if (vcc && entry && !vcc->entry->itf) {
+	if (!entry->svc) {
+	    diag(COMPONENT,DIAG_ERROR,"attempt to overwrite PVC for IP "
+	      "%d.%d.%d.%d",ipp[0],ipp[1],ipp[2],ipp[3]);
+	    return;
+	}
+	STOP_TIMER(vcc->entry);
+	Q_REMOVE(unknown_incoming,vcc->entry);
+	free(vcc->entry);
+	vcc->entry = entry;
+	Q_INSERT_HEAD(entry->vccs,vcc);
+	vcc = NULL;
+    }
+    if (entry || (vcc && (entry = vcc->entry))) {
+	if (!entry->itf) Q_REMOVE(unknown_incoming,entry);
+    }
+    else {
 	entry = alloc_t(ENTRY);
 	entry->svc = 1;
 	entry->state = as_none;
@@ -422,13 +453,12 @@ static void learn(VCC *vcc,unsigned long ip,struct sockaddr_atmsvc *addr)
     entry->addr->sas_txtp.class = ATM_UBR;
     entry->addr->sas_txtp.max_sdu = RFC1483LLC_LEN+RFC1626_MTU;
     entry->addr->sas_rxtp = entry->addr->sas_txtp;
-    if (entry->state != as_valid) {
-	for (walk = entry->vccs; walk; walk = walk->next)
-	    if (!walk->connecting)
-		if (set_ip(walk->fd,ip) < 0)
-		    diag(COMPONENT,DIAG_ERROR,"set_ip: %s",strerror(errno));
-	if (!entry->vccs && itf->arp_srv) connect_me(entry);
-    }
+    for (walk = entry->vccs; walk; walk = walk->next)
+	if (!walk->connecting)
+	    if (set_ip(walk->fd,ip) < 0)
+		diag(COMPONENT,DIAG_ERROR,"set_ip: %s",strerror(errno));
+    if (entry->state != as_valid && !entry->vccs && itf->arp_srv)
+	connect_me(entry);
     if ((entry->flags & ATF_ARPSRV) || !(entry->flags & ATF_PERM))
 	START_TIMER(entry,REVAL);
     entry->state = as_valid;
@@ -642,8 +672,10 @@ static int ioctl_set_pvc(ITF *itf,unsigned long ip,
     int fd,result;
 
     if (lookup_ip(itf,ip)) return -EEXIST; 
-    if ((fd = connect_vcc((struct sockaddr *) addr)) < 0) return fd;
+    if ((fd = connect_vcc((struct sockaddr *) addr,0)) < 0) return fd;
     if ((result = set_ip(fd,ip)) < 0) return result;
+    if (flags & ATF_NULL)
+	if ((result = set_encap(fd,0)) < 0) return result;
     entry = alloc_t(ENTRY);
     entry->state = as_valid;
     entry->svc = 0;
@@ -654,6 +686,7 @@ static int ioctl_set_pvc(ITF *itf,unsigned long ip,
     entry->itf = itf;
     entry->vccs = NULL;
     vcc = alloc_t(VCC);
+    vcc->active = 1;
     vcc->connecting = 0;
     vcc->fd = fd;
     vcc->entry = entry;
@@ -782,8 +815,7 @@ void vcc_failed(VCC *vcc)
 
     diag(COMPONENT,DIAG_DEBUG,"failed VCC 0x%08x",(unsigned long) vcc);
     Q_REMOVE(vcc->entry->vccs,vcc);
-    if (!(vcc->entry->flags & ATF_ARPSRV) || vcc->entry->vccs != vcc ||
-      vcc->next) {
+    if (!(vcc->entry->flags & ATF_ARPSRV) || vcc->entry->vccs) {
 	/* VCC is already closed */
 	vcc_detach(vcc->entry);
 	free(vcc);
@@ -794,9 +826,9 @@ void vcc_failed(VCC *vcc)
 	if (entry != vcc->entry && entry->state == as_resolv)
 	    if (entry->vccs || (entry->flags & ATF_PERM))
 		entry->state = as_invalid;
-	    else discard_entry(vcc->entry);
+	    else discard_entry(entry);
     }
-    START_TIMER(entry,RETRY);
+    START_TIMER(vcc->entry,RETRY);
     free(vcc);
 }
 
