@@ -1,6 +1,6 @@
 /* atmarpd.c - ATMARP demon */
  
-/* Written 1995 by Werner Almesberger, EPFL-LRC */
+/* Written 1995,1996 by Werner Almesberger, EPFL-LRC */
  
 
 #include <stdio.h>
@@ -25,7 +25,7 @@
 
 
 ITF *itfs = NULL;
-VCC *pending = NULL;
+ENTRY *unknown_incoming = NULL;
 int debug;
 
 
@@ -40,6 +40,7 @@ int main(int argc,char **argv)
 {
     int c,background;
 
+    set_application("atmarpd");
     set_verbosity(NULL,DIAG_INFO);
     background = 0;
     while ((c = getopt(argc,argv,"bdnp")) != EOF)
