@@ -1,6 +1,6 @@
 /* io.c - I/O operations */
  
-/* Written 1995 by Werner Almesberger, EPFL-LRC */
+/* Written 1995,1996 by Werner Almesberger, EPFL-LRC */
  
 
 #include <stdio.h>
@@ -15,6 +15,7 @@
 #include <net/if.h>
 #include <netinet/in.h>
 #include <linux/atm.h>
+#include <linux/atmclip.h> /* for CLIP_DEFAULT_IDLETIMER */
 #include <linux/atmarp.h>
 
 #include "atm.h"
@@ -56,10 +57,17 @@ void open_all(void)
     blli.l2_proto = ATM_L2_ISO8802;
     blli.l3_proto = ATM_L3_NONE;
     blli.next = NULL;
-    if (bind(incoming,(struct sockaddr *) &addr,sizeof(addr)) < 0)
-	diag(COMPONENT,DIAG_FATAL,"bind: %s",strerror(errno));
-    if (listen(incoming,5) < 0)
-	diag(COMPONENT,DIAG_FATAL,"listen: %s",strerror(errno));
+    if (bind(incoming,(struct sockaddr *) &addr,sizeof(addr)) >= 0) {
+	if (listen(incoming,5) < 0)
+	    diag(COMPONENT,DIAG_FATAL,"listen: %s",strerror(errno));
+    }
+    else if (errno != EUNATCH)
+	    diag(COMPONENT,DIAG_FATAL,"bind: %s",strerror(errno));
+	else {
+	    diag(COMPONENT,DIAG_WARN,"SVCs are not available",strerror(errno));
+	    (void) close(incoming);
+	    incoming = -1;
+	}
     if ((inet = socket(PF_INET,SOCK_DGRAM,0)) < 0)
 	diag(COMPONENT,DIAG_FATAL,"socket: %s",strerror(errno));
 }
@@ -123,7 +131,7 @@ static void recv_kernel(void)
 
 void close_all(void)
 {
-    (void) close(incoming);
+    if (incoming >= 0) (void) close(incoming);
     (void) close(kernel); /* may get major complaints from the kernel ... */
     (void) close(inet);
 }
@@ -161,6 +169,7 @@ static void recv_vcc(VCC *vcc)
     }
     if (size < 0) {
 	diag(COMPONENT,DIAG_ERROR,"read vcc: %s",strerror(errno));
+	disconnect_vcc(vcc);
 	return;
     }
     if (debug) {
@@ -185,12 +194,13 @@ static void accept_new(void)
 	diag(COMPONENT,DIAG_ERROR,"accept: %s",strerror(errno));
 	return;
     }
-    if (ioctl(fd,ATMARP_MKIP,0) < 0) {
+    if (ioctl(fd,ATMARP_MKIP,CLIP_DEFAULT_IDLETIMER) < 0) {
         diag(COMPONENT,DIAG_ERROR,"ioctl ATMARP_MKIP: %s",strerror(errno));
         (void) do_close(fd);
         return;
     }
     vcc = alloc_t(VCC);
+    vcc->active = 0;
     vcc->connecting = 0;
     vcc->fd = fd;
 #if 0
@@ -213,6 +223,7 @@ static void accept_new(void)
     entry->flags = 0;
     entry->vccs = NULL;
     entry->itf = NULL;
+    Q_INSERT_HEAD(unknown_incoming,entry);
     vcc->entry = entry;
     Q_INSERT_HEAD(entry->vccs,vcc);
     if (atm2text(buffer,MAX_ATM_ADDR_LEN+1,(struct sockaddr *) &addr,
@@ -222,10 +233,11 @@ static void accept_new(void)
 }
 
 
-int connect_vcc(struct sockaddr *remote)
+int connect_vcc(struct sockaddr *remote,int timeout)
 {
     int fd,flags;
 
+    if (remote->sa_family == AF_ATMSVC && incoming < 0) return -EUNATCH;
     if ((fd = socket(remote->sa_family,SOCK_DGRAM,ATM_AAL5)) < 0) {
 	diag(COMPONENT,DIAG_ERROR,"socket: %s",strerror(errno));
 	return -errno;
@@ -265,11 +277,12 @@ int connect_vcc(struct sockaddr *remote)
     }
     /* PVC connect never blocks */
     if (connect(fd,remote,remote->sa_family == AF_ATMPVC ?
-      sizeof(struct sockaddr_atmpvc) : sizeof(struct sockaddr_atmsvc)) < 0) {
-	diag(COMPONENT,DIAG_ERROR,"[1]connect: %s",strerror(errno));
-	if (errno != EINPROGRESS) return -errno;
-    }
-    if (ioctl(fd,ATMARP_MKIP,0) < 0) {
+      sizeof(struct sockaddr_atmpvc) : sizeof(struct sockaddr_atmsvc)) < 0)
+	if (errno != EINPROGRESS) {
+	    diag(COMPONENT,DIAG_ERROR,"[1]connect: %s",strerror(errno));
+	    return -errno;
+	}
+    if (ioctl(fd,ATMARP_MKIP,timeout) < 0) {
         diag(COMPONENT,DIAG_ERROR,"ioctl ATMARP_MKIP: %s",strerror(errno));
         (void) do_close(fd);
         return -errno;
@@ -287,6 +300,15 @@ int set_ip(int fd,int ip)
 }
 
 
+int set_encap(int fd,int mode)
+{
+    if (ioctl(fd,ATMARP_ENCAP,mode) >= 0) return 0;
+    diag(COMPONENT,DIAG_ERROR,"ioctl ATMARP_ENCAP: %s",strerror(errno));
+    (void) do_close(fd);
+    return -errno;
+}
+
+
 static void complete_connect(VCC *vcc)
 {
     struct sockaddr_atmsvc dummy;
@@ -307,9 +329,9 @@ static void complete_connect(VCC *vcc)
 
 void poll_loop(void)
 {
-    ITF *itf;
-    ENTRY *entry;
-    VCC *vcc;
+    ITF *itf,*next_itf;
+    ENTRY *entry,*next_entry;
+    VCC *vcc,*next_vcc;
     int fds,ret;
 
     gettimeofday(&now,NULL);
@@ -317,7 +339,7 @@ void poll_loop(void)
 	FD_ZERO(&rset);
 	FD_ZERO(&cset);
 	FD_SET(kernel,&rset);
-	FD_SET(incoming,&rset);
+	if (incoming >= 0) FD_SET(incoming,&rset);
 	fds = (kernel > incoming ? kernel : incoming)+1;
 	for (itf = itfs; itf; itf = itf->next)
 	    for (entry = itf->table; entry; entry = entry->next)
@@ -326,9 +348,13 @@ void poll_loop(void)
 		    else FD_SET(vcc->fd,&rset);
 		    if (vcc->fd >= fds) fds = vcc->fd+1;
 		}
-	for (vcc = pending; vcc; vcc = vcc->next) {
-	    FD_SET(vcc->fd,&rset);
-	    if (vcc->fd >= fds) fds = vcc->fd+1;
+	for (entry = unknown_incoming; entry; entry = entry->next) {
+	    if (!entry->vccs || entry->vccs->next) {
+		diag(COMPONENT,DIAG_ERROR,"internal error: bad unknown entry");
+		continue;
+	    }
+	    FD_SET(entry->vccs->fd,&rset);
+	    if (entry->vccs->fd >= fds) fds = entry->vccs->fd+1;
 	}
 	ret = select(fds,&rset,&cset,&cset,next_timer());
 	if (ret < 0) {
@@ -338,15 +364,23 @@ void poll_loop(void)
 	    diag(COMPONENT,DIAG_DEBUG,"----------");
 	    gettimeofday(&now,NULL);
 	    if (FD_ISSET(kernel,&rset)) recv_kernel();
-	    if (FD_ISSET(incoming,&rset)) accept_new();
-	    for (itf = itfs; itf; itf = itf->next)
-		for (entry = itf->table; entry; entry = entry->next)
-		    for (vcc = entry->vccs; vcc; vcc = vcc->next)
+	    if (incoming >= 0 && FD_ISSET(incoming,&rset)) accept_new();
+	    for (itf = itfs; itf; itf = next_itf) {
+		next_itf = itf->next;
+		for (entry = itf->table; entry; entry = next_entry) {
+		    next_entry = entry->next;
+		    for (vcc = entry->vccs; vcc; vcc = next_vcc) {
+			next_vcc = vcc->next;
 			if (FD_ISSET(vcc->fd,&rset)) recv_vcc(vcc);
 			else if (FD_ISSET(vcc->fd,&cset))
 				complete_connect(vcc);
-	    for (vcc = pending; vcc; vcc = vcc->next)
-		if (FD_ISSET(vcc->fd,&rset)) recv_vcc(vcc);
+		    }
+		}
+	    }
+	    for (entry = unknown_incoming; entry; entry = next_entry) {
+		next_entry = entry->next;
+		if (FD_ISSET(entry->vccs->fd,&rset)) recv_vcc(entry->vccs);
+	    }
 	    expire_timers();
 	      /* expire timers after handling messges to make sure we don't
 		 time out unnecessarily because of scheduling delays */
@@ -376,7 +410,7 @@ void send_packet(int fd,void *data,int length)
 int ip_itf_info(int number,unsigned long *ip,unsigned long *netmask)
 {
     struct ifreq req;
-    unsigned char *p;
+    unsigned char *p1,*p2;
 
     sprintf(req.ifr_ifrn.ifrn_name,"atm%d",number);
     if (ioctl(inet,SIOCGIFADDR,&req) < 0) {
@@ -390,10 +424,10 @@ int ip_itf_info(int number,unsigned long *ip,unsigned long *netmask)
     }
     *netmask = ((struct sockaddr_in *) &req.ifr_ifru.ifru_netmask)->
       sin_addr.s_addr;
-    p = (unsigned char *) ip;
-printf("ip %d.%d.%d.%d",p[0],p[1],p[2],p[3]);
-    p = (unsigned char *) netmask;
-printf(" mask %d.%d.%d.%d\n",p[0],p[1],p[2],p[3]);
+    p1 = (unsigned char *) ip;
+    p2 = (unsigned char *) netmask;
+    diag(COMPONENT,DIAG_DEBUG,"ip %d.%d.%d.%d mask %d.%d.%d.%d\n",
+      p1[0],p1[1],p1[2],p1[3],p2[0],p2[1],p2[2],p2[3]);
     return 0;
 }
 
@@ -404,6 +438,7 @@ int atm_itf_info(struct sockaddr_atmsvc *addr)
 
     req.number = 0; /* @@@ */
     req.arg = addr;
+    req.length = sizeof(*addr);
     if (ioctl(kernel,SIOCGIFATMADDR,&req) >= 0) return 0;
     diag(COMPONENT,DIAG_FATAL,"ioctl SIOCGIFATMADDR: %s",strerror(errno));
     return -1;
