@@ -9,7 +9,9 @@
 #include "table.h"
 
 ITF *lookup_itf(int number);
-int itf_up(int number);
+ITF *lookup_itf_by_ip(unsigned long ip);
+int itf_create(int number);
+void itf_up(int number);
 void itf_down(int number);
 
 #endif
