@@ -1,6 +1,6 @@
 /* table.c - ATMARP table */
  
-/* Written 1995 by Werner Almesberger, EPFL-LRC */
+/* Written 1995,1996 by Werner Almesberger, EPFL-LRC */
  
 
 #include <stdio.h>
@@ -69,23 +69,18 @@ static void dump_vcc(VCC *vcc)
 	  A2T_PRETTY) < 0) strcpy(buffer,"<atm2text error>");
     }
     diag(COMPONENT,DIAG_DEBUG,"  %s%s",buffer,vcc->connecting ? ", connecting"
-      : "");
+      : !vcc->entry->svc ? "" : vcc->active ? " (active)" : " (passive)");
 }
 
 
-void dump_itf(ITF *itf)
+static void dump_entries(ENTRY *list)
 {
     ENTRY *entry;
     VCC *vcc;
     char buffer[MAX_ATM_ADDR_LEN+1];
     unsigned char *ipp;
 
-    ipp = (unsigned char *) &itf->local_ip;
-    if (atm2text(buffer,MAX_ATM_ADDR_LEN+1,(struct sockaddr *) &itf->local_addr,
-      A2T_PRETTY) < 0) strcpy(buffer,"<unknown>");
-    diag(COMPONENT,DIAG_DEBUG,"----- Itf %d (%d.%d.%d.%d, %s) -----",
-      itf->number,ipp[0],ipp[1],ipp[2],ipp[3],buffer);
-    for (entry = itf->table; entry ; entry = entry->next) {
+    for (entry = list; entry ; entry = entry->next) {
 	if (!entry->addr) strcpy(buffer,"<none>");
 	else if (atm2text(buffer,MAX_ATM_ADDR_LEN+1,
 	      (struct sockaddr *) entry->addr,A2T_PRETTY) < 0)
@@ -99,13 +94,26 @@ void dump_itf(ITF *itf)
 }
 
 
+void dump_itf(ITF *itf)
+{
+    char buffer[MAX_ATM_ADDR_LEN+1];
+    unsigned char *ipp;
+
+    ipp = (unsigned char *) &itf->local_ip;
+    if (atm2text(buffer,MAX_ATM_ADDR_LEN+1,(struct sockaddr *) &itf->local_addr,
+      A2T_PRETTY) < 0) strcpy(buffer,"<unknown>");
+    diag(COMPONENT,DIAG_DEBUG,"----- Itf %d (%d.%d.%d.%d, %s) -----",
+      itf->number,ipp[0],ipp[1],ipp[2],ipp[3],buffer);
+    dump_entries(itf->table);
+}
+
+
 void dump_all(void)
 {
     ITF *itf;
-    VCC *vcc;
 
     for (itf = itfs; itf; itf = itf->next) dump_itf(itf);
-    diag(COMPONENT,DIAG_DEBUG,"----- Pending connections -----");
-    for (vcc = pending; vcc; vcc = vcc->next) dump_vcc(vcc);
+    diag(COMPONENT,DIAG_DEBUG,"----- Unknown incoming connections -----");
+    dump_entries(unknown_incoming);
     diag(COMPONENT,DIAG_DEBUG,"----- End of dump -----");
 }
