@@ -60,7 +60,8 @@ AsnOid *get_esi(int fd)
 
   req.number = 0;
   req.arg = esi;
-  
+  req.length = ESI_LEN;
+
   if(ioctl(fd, ATM_GETESI, &req) < 0)
     diag(COMPONENT, DIAG_FATAL, "ioctl ATM_GETESI: %s", strerror(errno));
   
@@ -131,6 +132,7 @@ void add_nsap(AsnOid *netprefix, AsnOid *esi)
 
   req.number = atm_itf;
   req.arg = &addr;
+  req.length = sizeof(addr);
 
   if(ioctl(fd, SIOCSIFATMADDR, &req) < 0)
     diag(COMPONENT, DIAG_FATAL, "ioctl SIOCSIFATMADDR: %s", strerror(errno));
