@@ -1,6 +1,6 @@
 /* atmarp.c - RFC1577 ATMARP control */
  
-/* Written 1995 by Werner Almesberger, EPFL-LRC */
+/* Written 1995,1996 by Werner Almesberger, EPFL-LRC */
 
 
 #include <stdlib.h>
@@ -25,7 +25,7 @@
 
 static void usage(const char *name)
 {
-    fprintf(stderr,"usage: %s -c atmN\n",name);
+    fprintf(stderr,"usage: %s -c [atmN]\n",name);
     fprintf(stderr,"%6s %s -s ip_addr [itf.]vpi.vci [temp] [null]\n","",name);
     fprintf(stderr,"%6s %s -s ip_addr atm_addr [temp] [arpsrv]\n","",name);
     fprintf(stderr,"%6s %s -d ip_addr\n","",name);
@@ -37,7 +37,7 @@ int main(int argc,char **argv)
 {
     struct hostent *hostent;
     struct atmarpreq req;
-    int c,op,s,i,num;
+    int c,op,s,i,req_num,num;
     char *here,*end;
 
     op = 0;
@@ -60,25 +60,27 @@ int main(int argc,char **argv)
 	}
     switch (op) {
 	case SIOCMKCLIP:
-	    if (argc != optind+1) usage(argv[0]);
+	    if (argc == optind) req_num = -1;
+	    else {
+		if (argc != optind+1) usage(argv[0]);
+		here = argv[optind];
+		if (strlen(here) > 3 && !strncmp(here,"atm",3)) here += 3;
+		else usage(argv[0]);
+		req_num = strtoul(here,&end,10);
+		if (*end || (here[0] == '0' && here[1])) {
+		    usage(argv[0]);
+		    return 1;
+		}
+	    }
 	    if ((s = socket(PF_ATMSVC,SOCK_DGRAM,0)) < 0) {
 		perror("socket");
 		return 1;
 	    }
-	    here = argv[optind];
-	    if (strlen(here) > 3 && !strncmp(here,"atm",3)) here += 3;
-#if 1
-	    else usage(argv[0]);
-#endif
-	    num = strtoul(here,&end,10);
-	    if (*end || (here[0] == '0' && here[1])) {
-		usage(argv[0]);
-		return 1;
-	    }
-	    if (ioctl(s,op,num) < 0) {
+	    if ((num = ioctl(s,op,req_num)) < 0) {
 		perror("ioctl");
 		return 1;
 	    }
+	    if (req_num == -1) printf("atm%d\n",num);
 	    return 0;
 	case SIOCSARP:
 	    if (argc < optind+2) usage(argv[0]);
