@@ -1,6 +1,6 @@
 /* atmlib.h - Functions useful for ATM applications */
  
-/* Written 1995 by Werner Almesberger, EPFL-LRC */
+/* Written 1995,1996 by Werner Almesberger, EPFL-LRC */
  
 
 #ifndef ATMLIB_H
@@ -31,5 +31,6 @@ int text2atm(const char *text,struct sockaddr *addr,int length,int flags);
 int atm2text(char *buffer,int length,const struct sockaddr *addr,int flags);
 int atm_equal(const struct sockaddr_atmsvc *a,const struct sockaddr_atmsvc *b,
   int len,int flags);
+int sdu2cell(int s,int sizes,const int *sdu_size,int *num_sdu);
 
 #endif
