@@ -13,6 +13,10 @@
 #include "atm.h"
 
 
+#define TRY_OTHER -2
+#define FATAL	  -1 /* must be -1 */
+
+
 static int try_pvc(const char *text,struct sockaddr_atmpvc *addr,int flags)
 {
     int part[3];
@@ -21,44 +25,45 @@ static int try_pvc(const char *text,struct sockaddr_atmpvc *addr,int flags)
     part[0] = part[1] = part[2] = 0;
     i = 0;
     while (1) {
-	if (!*text) return -1; /* empty or ends with a dot */
-	if (i == 3) return 1; /* too long */
+	if (!*text) return FATAL; /* empty or ends with a dot */
+	if (i == 3) return TRY_OTHER; /* too long */
 	if (isdigit(*text)) {
-	    if (*text == '0' && isdigit(text[1])) return 1;
+	    if (*text == '0' && isdigit(text[1])) return TRY_OTHER;
 		/* no leading zeroes */
 	    do {
-		if (part[i] > INT_MAX/10) return 1; /* number too big */
+		if (part[i] > INT_MAX/10) return TRY_OTHER;/* number too big */
 		part[i] = part[i]*10+*text++-'0';
 	    }
 	    while (isdigit(*text));
 	    i++;
 	    if (!*text) break;
-	    if (*text++ != '.') return 1; /* non-PVC character */
+	    if (*text++ != '.') return TRY_OTHER; /* non-PVC character */
 	    continue;
 	}
 	if (*text == '*') {
-	    if (!(flags & T2A_WILDCARD)) return -1; /* not allowed */
+	    if (!(flags & T2A_WILDCARD)) return FATAL; /* not allowed */
 	    part[i++] = ATM_ITF_ANY; /* all *_ANY have the same value */
 	}
 	else {
-	    if (*text != '?') return 1; /* invalid character */
-	    if (!(flags & T2A_UNSPEC)) return -1; /* not allowed */
+	    if (*text != '?') return TRY_OTHER; /* invalid character */
+	    if (!(flags & T2A_UNSPEC)) return FATAL; /* not allowed */
 	    part[i++] = ATM_VPI_UNSPEC; /* all *_UNSPEC have the same
 					   value */
 	}
 	if (!*++text) break;
-	if (*text++ != '.') return -1; /* dot required */
+	if (*text++ != '.') return FATAL; /* dot required */
     }
-    if (i < 2) return 1; /* no dots */
+    if (i < 2) return TRY_OTHER; /* no dots */
     if (i == 2) {
 	part[2] = part[1];
 	part[1] = part[0];
 	part[0] = 0; /* default interface */
     }
-    if (part[0] > SHRT_MAX || part[2] > ATM_MAX_VCI) return 1; /* too big */
+    if (part[0] > SHRT_MAX || part[2] > ATM_MAX_VCI)
+	return TRY_OTHER; /* too big */
     if (part[1] > (flags & T2A_NNI ? ATM_MAX_VPI_NNI : ATM_MAX_VPI))
-	return 1; /* too big */
-    if (part[0] == ATM_VPI_UNSPEC) return -1; /* bad */
+	return TRY_OTHER; /* too big */
+    if (part[0] == ATM_VPI_UNSPEC) return FATAL; /* bad */
     addr->sap_family = AF_ATMPVC;
     addr->sap_addr.itf = part[0];
     addr->sap_addr.vpi = part[1];
@@ -73,6 +78,7 @@ static int do_try_nsap(const char *text,struct sockaddr_atmsvc *addr,int flags)
 {
     const char *walk;
     int count,pos,dot;
+    int offset,len;
     char value;
 
     count = dot = 0;
@@ -82,11 +88,14 @@ static int do_try_nsap(const char *text,struct sockaddr_atmsvc *addr,int flags)
 	    dot = 1;
 	}
 	else if (*text != '.') break;
-	    else if (!dot) return -1; /* two dots in a row */
+	    else if (!dot) return FATAL; /* two dots in a row */
 		else dot = 0;
-    if (*walk != ':') pos = 0;
+    if (*walk != ':') {
+	pos = 0;
+	offset = 0;
+    }
     else {
-	if (!dot || *text == '0') return -1;
+	if (!dot || *text == '0') return FATAL;
 	addr->sas_addr.prv[0] = ATM_AFI_E164;
 	addr->sas_addr.prv[1] = 0;
 	memset(addr->sas_addr.prv+1,0,8);
@@ -101,10 +110,11 @@ static int do_try_nsap(const char *text,struct sockaddr_atmsvc *addr,int flags)
 	addr->sas_addr.prv[8] |= 0xf;
 	text++;
 	pos++;
+	offset = 72;
     }
     for (dot = 0; *text; text++)
 	if (isxdigit(*text)) {
-	    if (pos == ATM_ESA_LEN*2) return 1; /* too long */
+	    if (pos == ATM_ESA_LEN*2) return TRY_OTHER; /* too long */
 	    value = isdigit(*text) ? *text-'0' : (islower(*text) ?
 	      toupper(*text) : *text)-'A'+10;
 	    if (pos & 1) addr->sas_addr.prv[pos >> 1] |= value;
@@ -112,14 +122,28 @@ static int do_try_nsap(const char *text,struct sockaddr_atmsvc *addr,int flags)
 	    pos++;
 	    dot = 1;
 	}
-	else if (*text != '.') return 1;
-	    else {
-		if (!dot) return -1; /* two dots in a row */
-		dot = 0;
-	    }
-    if (!dot) return -1;
-    if (!*addr->sas_addr.prv) return 1; /* no leading zeroes */
-    return pos != ATM_ESA_LEN*2;
+	else 
+	    if (*text == '/' && (flags & T2A_WILDCARD)) break;
+	    else if (*text != '.') return TRY_OTHER;
+		else {
+		    if (!dot) return FATAL; /* two dots in a row */
+		    dot = 0;
+		}
+    if (!dot) return FATAL;
+    if (pos > 1 && !*addr->sas_addr.prv)
+	return TRY_OTHER; /* no leading zeroes */
+    if (!*text)
+	return pos != ATM_ESA_LEN*2 ? TRY_OTHER : ATM_ESA_LEN*2;
+	  /* handle bad length */
+    len = 0;
+    while (*++text) {
+	if (!isdigit(*text)) return -1; /* non-digit in length */
+	if (len >= pos*4) return -1; /* too long */
+	len = len*10+*text-'0';
+    }
+    if (len > 7 && addr->sas_addr.prv[0] != ATM_AFI_E164) offset = 72;
+    if (len < offset) return FATAL;
+    return len > pos*4 ? TRY_OTHER : len;
 }
 
 
@@ -128,40 +152,45 @@ static int try_nsap(const char *text,struct sockaddr_atmsvc *addr,int flags)
     int result;
 
     result = do_try_nsap(text,addr,flags);
-    if (result) return result;
+    if (result < 0) return result;
     addr->sas_family = AF_ATMSVC;
     addr->sas_addr.bhli.hl_type = ATM_HL_NONE;
     addr->sas_addr.blli = NULL;
     *addr->sas_addr.pub = 0;
-    return 0;
+    return result;
 }
 
 
 static int try_e164(const char *text,struct sockaddr_atmsvc *addr,int flags)
 {
-    int i,dot;
+    int i,dot,result;
 
     if (*text == ':') text++;
     for (i = dot = 0; *text; text++)
 	if (isdigit(*text)) {
-	    if (i == ATM_E164_LEN) return 1; /* too long */
+	    if (i == ATM_E164_LEN) return TRY_OTHER; /* too long */
 	    addr->sas_addr.pub[i++] = *text;
 	    dot = 1;
 	}
 	else if (*text != '.') break;
 	    else {
-		if (!dot) return -1; /* two dots in a row */
+		if (!dot) return FATAL; /* two dots in a row */
 		dot = 0;
 	    }
-    if (!dot) return -1;
+    if (!dot) return FATAL;
+    addr->sas_addr.pub[i] = 0;
     *addr->sas_addr.prv = 0;
+    result = 0;
     if (*text)
-	if (*text++ != '+') return 1;
-	else if (do_try_nsap(text,addr,flags)) return -1;
+	if (*text++ != '+') return TRY_OTHER;
+	else {
+	    result = do_try_nsap(text,addr,flags);
+	    if (result < 0) return FATAL;
+	}
     addr->sas_family = AF_ATMSVC;
     addr->sas_addr.bhli.hl_type = ATM_HL_NONE;
     addr->sas_addr.blli = NULL;
-    return 0;
+    return result;
 }
 
 
@@ -174,14 +203,14 @@ int text2atm(const char *text,struct sockaddr *addr,int length,int flags)
     if (length < sizeof(struct sockaddr_atmpvc)) return -1;
     if (flags & T2A_PVC) {
 	result = try_pvc(text,(struct sockaddr_atmpvc *) addr,flags);
-	if (result <= 0) return result;
+	if (result != TRY_OTHER) return result;
     }
     if (!(flags & T2A_SVC)) return -1;
     if (length < sizeof(struct sockaddr_atmsvc)) return -1;
     result = try_nsap(text,(struct sockaddr_atmsvc *) addr,flags);
-    if (result <= 0) return result;
+    if (result != TRY_OTHER) return result;
     result = try_e164(text,(struct sockaddr_atmsvc *) addr,flags);
-    if (result <= 0) return result;
+    if (result != TRY_OTHER) return result;
     if (!(flags & T2A_NAME)) return -1;
     /* @@@ name resolution */
     return -1;
