@@ -1,6 +1,6 @@
 /* atmdiag.c - ATM diagnostics */
 
-/* Written 1995 by Werner Almesberger, EPFL-LRC */
+/* Written 1995,1996 by Werner Almesberger, EPFL-LRC */
 
 
 #include <stdlib.h>
@@ -32,6 +32,7 @@ static void display(int s,int itf,int zero)
 
     req.number = itf;
     req.arg = &stats;
+    req.length = sizeof(stats);
     if (ioctl(s,zero ? ATM_GETSTATZ : ATM_GETSTAT,&req) < 0) {
 	perror(zero ? "ioctl ATM_GETSTATZ" : "ioctl ATM_GETSTAT");
 	exit(1);
