@@ -1,6 +1,6 @@
 /* sonetdiag.c - SONET diagnostics */
 
-/* Written 1995 by Werner Almesberger, EPFL-LRC */
+/* Written 1995,1996 by Werner Almesberger, EPFL-LRC */
 
 
 #include <stdlib.h>
@@ -79,11 +79,13 @@ int main(int argc,char **argv)
     if (error) return 1;
     if (!set && !clear) {
 	req.arg = &stats;
+	req.length = sizeof(stats);
 	if (ioctl(s,zero ? SONET_GETSTATZ : SONET_GETSTAT,&req) < 0) {
 	    perror(zero ? "ioctl SONET_GETSTATZ" : "ioctl SONET_GETSTAT");
 	    return 1;
 	}
 	req.arg = &set;
+	req.length = sizeof(set);
 	if (ioctl(s,SONET_GETDIAG,&req) < 0)
 	    if (errno != EINVAL) perror("ioctl SONET_GETDIAG");
 	if (stats.section_bip != -1)
@@ -116,11 +118,13 @@ int main(int argc,char **argv)
     else {
 	if (set) {
 	    req.arg = &set;
+	    req.length = sizeof(set);
 	    if (ioctl(s,SONET_SETDIAG,&req) < 0)
 		perror("ioctl SONET_SETDIAG");
 	}
 	if (clear) {
 	    req.arg = &clear;
+	    req.length = sizeof(clear);
 	    if (ioctl(s,SONET_CLRDIAG,&req) < 0)
 		perror("ioctl SONET_SETDIAG");
 	}
