@@ -1,6 +1,6 @@
 /* file.c - (source) file IO */
  
-/* Written 1995 by Werner Almesberger, EPFL-LRC */
+/* Written 1995,1996 by Werner Almesberger, EPFL-LRC */
  
 
 #include <stdlib.h>
@@ -26,7 +26,7 @@ typedef struct {
 int pc;
 
 
-static FILE *h_file,*c_file,*test_file;
+static FILE *h_file,*c_file,*test_file,*dump_file;
 static CODE_ITEM *buf;
 static int buf_size;
 
@@ -70,23 +70,49 @@ void to_test(const char *fmt,...)
 }
 
 
-void open_files(void)
+void to_dump(const char *fmt,...)
 {
-    h_file = fopen("q.out.h","w");
+    va_list ap;
+
+    va_start(ap,fmt);
+    if (vfprintf(dump_file,fmt,ap) == EOF) {
+	perror("write");
+	exit(1);
+    }
+    va_end(ap);
+}
+
+
+void open_files(const char *prefix)
+{
+    char name[100]; /* maximum name */
+
+    sprintf(name,"%s.out.h",prefix);
+    h_file = fopen(name,"w");
     if (!h_file) {
 	perror("creat");
 	exit(1);
     }
-    c_file = fopen("q.out.c","w");
+    sprintf(name,"%s.out.c",prefix);
+    c_file = fopen(name,"w");
     if (!c_file) {
 	perror("creat");
 	exit(1);
     }
-    test_file = fopen("q.test.c","w");
+    sprintf(name,"%s.test.c",prefix);
+    test_file = fopen(name,"w");
     if (!test_file) {
 	perror("creat");
 	exit(1);
     }
+    if (dump) {
+	sprintf(name,"%s.dump.c",prefix);
+	dump_file = fopen(name,"w");
+	if (!dump_file) {
+	    perror("creat");
+	    exit(1);
+	}
+    }
 }
 
 
@@ -104,6 +130,11 @@ void close_files(void)
 	perror("fclose");
 	exit(1);
     }
+    if (dump)
+	if (fclose(dump_file) == EOF) {
+	    perror("fclose");
+	    exit(1);
+	}
 }
 
 
