@@ -154,5 +154,11 @@
 #define ATM_RSN_IE_MISS		1 /* Information element missing */
 #define ATM_RSN_IE_INSUFF	2 /* Information element contents are not
 				     sufficient */
+/* Restart Indicator class values */
+
+#define ATM_RST_IND_VC		0 /* Indicated virtual channel */
+#define ATM_RST_ALL_VC		2 /* All virtual channels controlled by the
+				     Layer 3 entity which sends the RESTART
+				     message */
 
 #endif
