@@ -1,11 +1,27 @@
 /* qlib.c - run-time library */
  
-/* Written 1995 by Werner Almesberger, EPFL-LRC */
+/* Written 1995,1996 by Werner Almesberger, EPFL-LRC */
  
 
 int q_dump = 0;
 
 
+#ifdef DUMP_MODE
+
+#include <stdio.h>
+
+#include "common.h"
+#include "qlib.h"
+#include "op.h"
+
+
+void q_report(int severity,const char *msg,...)
+{
+}
+
+#endif
+
+
 static int q_test(unsigned char *table,int pos)
 {
     return !!(table[pos >> 3] & (1 << (pos & 7)));
@@ -85,7 +101,7 @@ static void q_put(unsigned char *table,int pos,int size,unsigned long value)
 }
 #endif
  
- 
+
 static void q_copy(unsigned char *src,int pos,unsigned char *dst,int size)
 {
     src += pos >> 3;
@@ -419,6 +435,37 @@ static int _q_parse(Q_DSC *dsc,unsigned char *buf,int size)
     while (1) {
 	q_report(Q_DEBUG,"%d(%d):",pc-parse,pos-buf);
 	switch (*pc++) {
+#ifdef DUMP_MODE
+	    case OP_DUMP:
+		{
+		    unsigned long value;
+		    int len;
+
+		    for (i = dump_fields[*pc].level; i; i--) printf("  ");
+		    printf("%s =",dump_fields[*pc++].name);
+		    len = *pc == OP_COPYVAR ? (end-pos)*8 : pc[3];
+		    if (len <= 32) {
+			const SYM_NAME *sym;
+
+			value = q_get(pos,pc[2] & 7,pc[3]);
+			if (!(sym = dump_fields[pc[-1]].sym))
+			    printf(" %ld (0x%lx)\n",value,value);
+			else {
+			    while (sym->name)
+				if (sym->value == value) break;
+				else sym++;
+			    if (sym->name) printf(" %s\n",sym->name);
+			    else printf(" %ld (0x%lx)\n",value,value);
+			}
+		    }
+		    else {
+			for (i = 0; i < len/8; i++)
+			    printf(" %02x",pos[pc[1]/8+i]);
+			putchar('\n');
+		    }
+		}
+		break;
+#endif
 	    case OP_COPY:
 		if (pos+*pc > end) {
 		    q_report(Q_FATAL,"not enough space (%d+%d > %d)",pos-buf,
@@ -442,14 +489,14 @@ static int _q_parse(Q_DSC *dsc,unsigned char *buf,int size)
 		    len = end-pos;
 		    if (len > pc[2]) len = pc[2];
 		    memcpy(dsc->data+pc[1]/8,pos,(size_t) len);
-		    q_report(Q_DEBUG,"len %d fot %d",len,*pc);
+		    q_report(Q_DEBUG,"len %d for %d",len,*pc);
 		    dsc->length[*pc] = len;
 		    pos += len;
 		    pc += 3;
 		    break;
 		}
 	    case OP_BEGIN_LEN:
-		if ( pos+*pc > end) {
+		if (pos+*pc > end) {
 		    q_report(Q_FATAL,"not enough space (%d+%d > %d)",pos-buf,
 		      *pc,end-buf);
 		    return -1;
@@ -581,3 +628,19 @@ int q_close(Q_DSC *dsc)
     if (dsc->length) free(dsc->length);
     return dsc->buffer ? size : 0;
 }
+
+
+#ifdef DUMP_MODE
+
+void main(void)
+{
+    unsigned char msg[5000]; /* should be large enough for that */
+    Q_DSC dsc;
+    int len,c;
+
+    len = 0;
+    while (scanf("%x",&c) == 1) msg[len++] = c;
+    q_open(&dsc,msg,len);
+}
+
+#endif
