@@ -1,10 +1,11 @@
 /* second.c - Phase II, table generation */
 
-/* Written 1995 by Werner Almesberger, EPFL-LRC */
+/* Written 1995,1996 by Werner Almesberger, EPFL-LRC */
 
 
 #include <stdlib.h>
 
+#include "common.h"
 #include "qgen.h"
 #include "file.h"
 
@@ -169,6 +170,55 @@ static void fields(FIELD *start,int group)
 }
 
 
+static void symbolic_names(FIELD *start)
+{
+    FIELD *walk;
+    TAG *scan;
+    VALUE_LIST *tag;
+
+    for (walk = start; walk; walk = walk->next) {
+	if (walk->value) {
+	    to_dump("static SYM_NAME dump_sym_%d[] = {\n",walk->seq);
+	    switch  (walk->value->type) {
+		case vt_id:
+		    to_dump("    { %s, \"%s\" },\n",walk->value->id,
+		      walk->value->id);
+		    break;
+		case vt_case:
+		case vt_multi:
+		    for (scan = walk->value->tags; scan; scan = scan->next) {
+			to_dump("    { %s, \"%s\" },\n",scan->value,
+			  scan->value);
+			for (tag = scan->more; tag; tag = tag->next)
+			    to_dump("    { %s, \"%s\" },\n",tag->value,
+			      tag->value);
+		    }
+		    break;
+		case vt_length:
+		    break;
+		default:
+		    abort();
+	    }
+	    to_dump("    { 0, NULL }\n};\n\n");
+	    switch  (walk->value->type) {
+		case vt_id:
+		    break;
+		case vt_case:
+		case vt_multi:
+		    for (scan = walk->value->tags; scan; scan = scan->next)
+			symbolic_names(scan->block);
+		    break;
+		case vt_length:
+		    symbolic_names(walk->value->block);
+		    break;
+		default:
+		    abort();
+	    }
+	}
+    }
+}
+
+
 void second(FIELD *def)
 {
     int i;
@@ -201,4 +251,9 @@ void second(FIELD *def)
     to_test("static int *unique[] = {\n");
     for (i = 0; i < unique; i++) to_test("    unique_%d,\n",i);
     to_test("    NULL\n};\n\n");
+    if (dump) {
+	to_dump("typedef struct {\n    unsigned long value;\n");
+	to_dump("    const char *name;\n} SYM_NAME;\n\n");
+	symbolic_names(def);
+    }
 }
