@@ -1,6 +1,6 @@
 /* io.c - I/O operations */
  
-/* Written 1995 by Werner Almesberger, EPFL-LRC */
+/* Written 1995,1996 by Werner Almesberger, EPFL-LRC */
  
 
 #include <stdio.h>
@@ -12,6 +12,7 @@
 #include <sys/socket.h>
 #include <sys/ioctl.h>
 #include <linux/atm.h>
+#include <linux/atmdev.h>
 #include <linux/atmsvc.h>
 
 #include "atmd.h"
@@ -194,6 +195,28 @@ static void close_signaling(void)
 }
 
 
+/* ----- addresses --------------------------------------------------------- */
+
+
+#define MAX_ADDRS 1024
+
+
+int get_addr(int itf,struct sockaddr_atmsvc *addr)
+{
+    struct atmif_sioc req;
+    struct sockaddr_atmsvc buffer[MAX_ADDRS];
+
+    req.number = itf;
+    req.arg = buffer;
+    req.length = sizeof(buffer);
+    if (ioctl(kernel,ATM_GETADDR,&req) < 0)
+	diag(COMPONENT,DIAG_FATAL,"ioctl ATM_GETADDR yields \"%s\"",
+	  strerror(errno));
+    if (req.length) *addr = *buffer;
+    return req.length/sizeof(*addr);
+}
+
+
 /* ----- common part ------------------------------------------------------- */
 
 
