@@ -1,6 +1,6 @@
 /* io.h - I/O operations */
  
-/* Written 1995 by Werner Almesberger, EPFL-LRC */
+/* Written 1995,1996 by Werner Almesberger, EPFL-LRC */
  
 
 #ifndef IO_H
@@ -16,4 +16,6 @@ void close_all(void);
 void to_kernel(struct atmsvc_msg *msg);
 void to_net(void *msg,int size);
 
+int get_addr(int itf,struct sockaddr_atmsvc *addr);
+
 #endif
