@@ -1,6 +1,6 @@
 /* kernel.c - Processing of incoming kernel messages */
  
-/* Written 1995 by Werner Almesberger, EPFL-LRC */
+/* Written 1995,1996 by Werner Almesberger, EPFL-LRC */
  
 
 #include <stdlib.h>
@@ -23,7 +23,7 @@
 #define COMPONENT "KERNEL"
 
 
-extern int vci;
+extern int assign_vci;
 
 static struct sockaddr_atmsvc local; /* local address */
 
@@ -48,6 +48,10 @@ static int send_setup(SOCKET *sock)
     q_assign(&dsc,QF_call_ref,call_ref);
     q_assign(&dsc,QF_msg_type,QMSG_SETUP);
     q_assign(&dsc,QF_aal_type,5); /* AAL 5 */
+#ifdef UNI30
+    q_assign(&dsc,QF_aal_mode,1); /* Message mode - LANE seems to really want
+				     this */
+#endif
     if (sock->remote->sas_txtp.max_sdu)
 	q_assign(&dsc,QF_fw_max_sdu,sock->remote->sas_txtp.max_sdu);
     if (sock->remote->sas_rxtp.max_sdu)
@@ -139,12 +143,11 @@ static int send_setup(SOCKET *sock)
 		    break;
 		case ATM_L3_TR9577:
 		    q_assign(&dsc,QF_ipi_high,blli->l3.tr9577.ipi >> 1);
-		    if ((blli->l3.tr9577.ipi >> 1) == 0x40)
-			q_assign(&dsc,QF_ipi_low,blli->l3.tr9577.ipi & 1);
-		    else if (!(blli->l3.tr9577.ipi & 1)) {
-			    q_write(&dsc,QF_oui,blli->l3.tr9577.snap,3);
-			    q_write(&dsc,QF_pid,blli->l3.tr9577.snap+3,2);
-			}
+		    q_assign(&dsc,QF_ipi_low,blli->l3.tr9577.ipi & 1);
+		    if (blli->l3.tr9577.ipi == NLPID_IEEE802_1_SNAP) {
+			q_write(&dsc,QF_oui,blli->l3.tr9577.snap,3);
+			q_write(&dsc,QF_pid,blli->l3.tr9577.snap+3,2);
+		    }
 		    break;
 		case ATM_L3_USER:
 		    q_assign(&dsc,QF_user_l3,blli->l3.user);
@@ -160,9 +163,9 @@ static int send_setup(SOCKET *sock)
 	  ATM_ESA_LEN);
     if (net) {
 	q_assign(&dsc,QF_vpi,0);
-	q_assign(&dsc,QF_vci,vci); /* @@@ */
+	q_assign(&dsc,QF_vci,assign_vci); /* @@@ */
 	sock->pvc.sap_addr.vpi = 0;
-	sock->pvc.sap_addr.vci = vci++;
+	sock->pvc.sap_addr.vci = assign_vci++;
     }
     size = q_close(&dsc);
     to_signaling(q_buffer,size);
@@ -335,20 +338,21 @@ static void dispatch(SOCKET *sock,struct atmsvc_msg *msg)
 }
 
 
-static void itf_notify(int itf,struct sockaddr_atmsvc *addr)
+static void itf_notify(int itf)
 {
+    struct sockaddr_atmsvc addr;
     char buf[MAX_ATM_ADDR_LEN+1];
 
-    if (!addr->sas_family) {
+    if (!get_addr(itf,&addr)) {
 	local.sas_family = 0;
 	diag(COMPONENT,DIAG_INFO,"Removed local ATM address");
 	return;
     }
-    if (atm2text(buf,MAX_ATM_ADDR_LEN+1,(struct sockaddr *) addr,A2T_PRETTY) <
+    if (atm2text(buf,MAX_ATM_ADDR_LEN+1,(struct sockaddr *) &addr,A2T_PRETTY) <
       0) diag(COMPONENT,DIAG_ERROR,"bad address from kernel");
     else {
 	diag(COMPONENT,DIAG_INFO,"Local ATM address set to %s",buf);
-	local = *addr;
+	local = addr;
     }
 }
 
@@ -360,7 +364,7 @@ void from_kernel(struct atmsvc_msg *msg,int size)
     int bllis,i;
 
     if (msg->type == as_itf_notify) {
-	itf_notify(msg->pvc.sap_addr.itf,&msg->local);
+	itf_notify(msg->pvc.sap_addr.itf);
 	return;
     }
     for (curr = sockets; curr; curr = curr->next)
