@@ -6,14 +6,13 @@
 #include <stdarg.h>
 #include <stdlib.h>
 #include <stdio.h>
+#include <string.h>
 
+#include "atmd.h"
 #include "q2931.h"
 #include "qlib.h"
 
-#include "diag.h"
-#include "common.h"
 #include "io.h"
-#include "timer.h"
 #include "proto.h"
 
 
@@ -66,7 +65,7 @@ SOCKET *new_sock(unsigned long id)
 }
 
 
-static void free_sap(struct sockaddr_atmsvc *sap)
+void free_sap(struct sockaddr_atmsvc *sap)
 {
     struct atm_blli *walk,*next;
 
@@ -118,7 +117,7 @@ struct sockaddr_atmsvc *alloc_sap(const struct sockaddr_atmsvc *sap)
     struct sockaddr_atmsvc *new_sap;
     struct atm_blli *walk,**here;
 
-    if (!*sap->sas_addr.prv && !*sap->sas_addr.pub) return NULL;
+    /*if (!*sap->sas_addr.prv && !*sap->sas_addr.pub) return NULL; @@@ */
     new_sap = alloc_t(struct sockaddr_atmsvc);
     *new_sap = *sap;
     here = &new_sap->sas_addr.blli;
@@ -130,6 +129,84 @@ struct sockaddr_atmsvc *alloc_sap(const struct sockaddr_atmsvc *sap)
 }
 
 
+static int match_blli(struct atm_blli *a,struct atm_blli *b)
+{
+    if (a->l2_proto != b->l2_proto || a->l3_proto != b->l3_proto) return 0;
+    switch (a->l3_proto) {
+	case ATM_L3_TR9577:
+	    if (a->l3.tr9577.ipi != b->l3.tr9577.ipi) return 0;
+	    if (a->l3.tr9577.ipi == NLPID_IEEE802_1_SNAP)
+		if (memcmp(a->l3.tr9577.snap,b->l3.tr9577.snap,5)) return 0;
+	    break;
+	case ATM_L3_USER:
+	    if (a->l3.user != b->l3.user) return 0;
+	    break;
+	default:
+    }
+    return 1;
+}
+
+
+static int match_sap(struct sockaddr_atmsvc *a,struct sockaddr_atmsvc *b)
+{
+    struct atm_blli *a_blli,*b_blli;
+    int length;
+
+/* @@@ check AAL */
+    if (a->sas_addr.bhli.hl_type != b->sas_addr.bhli.hl_type) return 0;
+    switch (a->sas_addr.bhli.hl_type) {
+	case ATM_HL_ISO:
+	case ATM_HL_USER:
+	    length = a->sas_addr.bhli.hl_length;
+	    if (length != b->sas_addr.bhli.hl_length) return 0;
+	    break;
+#ifdef UNI30
+	case ATM_HL_HLP:
+	    length = 4;
+	    break;
+#endif
+	case ATM_HL_VENDOR:
+	    length = 7;
+	    break;
+	default:
+	    length = 0;
+    }
+    if (length && memcmp(a->sas_addr.bhli.hl_info,b->sas_addr.bhli.hl_info,
+      length)) return 0;
+    a_blli = a->sas_addr.blli;
+    b_blli= b->sas_addr.blli;
+    if (!a_blli && !b_blli) return 1; /* redundant */
+    while (a_blli) {
+	while (b_blli) {
+	    if (match_blli(a_blli,b_blli)) return 1;
+	    b_blli = b_blli->next;
+	}
+	a_blli = a_blli->next;
+    }
+    return 0;
+}
+
+
+SOCKET *lookup_sap(struct sockaddr_atmsvc *sap)
+{
+    SOCKET *walk;
+
+    for (walk = sockets; walk; walk = walk->next)
+	if (walk->state == ss_listening) {
+	    if (!walk->local) break;
+	    if (sap->sas_txtp.max_sdu && sap->sas_txtp.max_sdu <
+	      walk->local->sas_txtp.max_sdu) continue;
+	    if (walk->local->sas_rxtp.max_sdu && sap->sas_rxtp.max_sdu >
+	      walk->local->sas_rxtp.max_sdu) continue;
+	    /* check PCR compat for CBR @@@ */
+	    if (!walk->local->sas_addr.bhli.hl_type &&
+	      !walk->local->sas_addr.blli) break;
+	    else if (match_sap(walk->local,sap)) break;
+	}
+    return walk;
+}
+
+
 const char *mid2name(unsigned char mid)
 {
     switch (mid) {
@@ -184,6 +261,7 @@ void send_kernel(unsigned long vcc,unsigned long listen_vcc,
     msg->listen_vcc = listen_vcc;
     msg->type = type;
     msg->reply = reply;
+    memset(&msg->remote,0,sizeof(msg->remote));
     if (sock) msg->pvc = sock->pvc;
     if (sock && sock->remote) {
 	msg->remote = *sock->remote;
@@ -195,15 +273,29 @@ void send_kernel(unsigned long vcc,unsigned long listen_vcc,
 }
 
 
-void send_release(SOCKET *sock,unsigned char reason)
+void send_release(SOCKET *sock,unsigned char reason,...)
 {
+    va_list ap;
     Q_DSC dsc;
     int size;
  
     q_create(&dsc,q_buffer,MAX_Q_MSG);
     q_assign(&dsc,QF_msg_type,QMSG_RELEASE);
     q_assign(&dsc,QF_call_ref,sock->call_ref);
-    /*reason */
+    q_assign(&dsc,QF_cause,reason);
+    va_start(ap,reason);
+    switch (reason) {
+	case ATM_CV_TIMER_EXP:
+	    {
+		char buf[4];
+
+		sprintf(buf,"%d",va_arg(ap,int));
+		q_write(&dsc,QF_timer,buf,3);
+		break;
+	    }
+	default:
+    }
+    va_end(ap);
     size = q_close(&dsc);
     to_signaling(q_buffer,size);
 }
