@@ -1,6 +1,6 @@
 /* proto.h - Common protocol functions and structures */
  
-/* Written 1995 by Werner Almesberger, EPFL-LRC */
+/* Written 1995,1996 by Werner Almesberger, EPFL-LRC */
  
 
 #ifndef PROTO_H
@@ -54,7 +54,8 @@ extern SOCKET *sockets;
 extern unsigned char q_buffer[];
 
 extern int net;
- 
+extern short sig_itf;
+
 
 #define msg2bllis(s) (((int) (s)-(int) sizeof(struct atmsvc_msg))/ \
   (int) sizeof(struct atm_blli)+1)
@@ -68,6 +69,9 @@ void send_kernel(unsigned long vcc,unsigned long listen_vcc,
   enum atmsvc_msg_type type,int reply,SOCKET *sock);
 void from_net(void *msg,int size);
 void to_signaling(void *msg,int size);
+void saal_failure(void);
+void saal_okay(void);
+void clear_all_calls_on_T309(void);
 
 SOCKET *new_sock(unsigned long id);
 void free_sock(SOCKET *sock);
