@@ -1,14 +1,8 @@
 /* q2931.c - Processing of incoming Q.2931 messages */
  
-/* Written 1995 by Werner Almesberger, EPFL-LRC */
+/* Written 1995,1996 by Werner Almesberger, EPFL-LRC */
  
 
-/*
- * TODO:
- * - concentrate sending of status
- */
-
-
 #include <stdarg.h>
 #include <stdio.h>
 #include <string.h>
@@ -30,8 +24,13 @@
 #define COMPONENT "Q2931"
 
 
+extern const char *cause_text[]; /* from mess.c */
+
+
+int assign_vci = 42; /* assigning VCIs from here on if acting as network */
+
 static Q_DSC in_dsc;
-int vci = 42; /* assigning VCIs from here on if acting as network */
+static TIMER *t309 = NULL;
 
 
 static void send_call_proceeding(SOCKET *sock)
@@ -44,10 +43,10 @@ static void send_call_proceeding(SOCKET *sock)
     q_assign(&dsc,QF_call_ref,sock->call_ref);
     if (net) {
 	q_assign(&dsc,QF_vpi,0);
-	q_assign(&dsc,QF_vci,vci); /* @@@ */
-	sock->pvc.sap_addr.itf = 0; /* @@@ use VPCI */
+	q_assign(&dsc,QF_vci,assign_vci); /* @@@ */
+	sock->pvc.sap_addr.itf = sig_itf; /* @@@ use VPCI */
 	sock->pvc.sap_addr.vpi = 0;
-	sock->pvc.sap_addr.vci = vci++;
+	sock->pvc.sap_addr.vci = assign_vci++;
     }
     size = q_close(&dsc);
     to_signaling(q_buffer,size);
@@ -182,16 +181,19 @@ static void setup_call(unsigned long call_ref)
     this->pvc.sap_txtp = sap->sas_txtp;
     this->pvc.sap_rxtp = sap->sas_rxtp;
     /* if (sock->local) *this->local->sas_addr = sock->local->sas_addr; ??? */
-    printf("AAL type %ld\n",q_fetch(&in_dsc,QF_aal_type));
+    diag(COMPONENT,DIAG_DEBUG,"AAL type %ld",q_fetch(&in_dsc,QF_aal_type));
     len = q_read(&in_dsc,QF_cdpn_esa,(void *) &buf,sizeof(buf));
-    printf("Addr len %d:",len);
-    for (i = 0; i < (len < 30 ? len : 30); i++) printf(" %02X",buf[i]);
-    putchar('\n');
+    if (debug) {
+	printf("Addr len %d:",len);
+	for (i = 0; i < (len < 30 ? len : 30); i++) printf(" %02X",buf[i]);
+	putchar('\n');
+    }
     if (q_present(&in_dsc,QF_vci)) {
-	this->pvc.sap_addr.itf = 0; /* @@@ use VPCI */
+	this->pvc.sap_addr.itf = sig_itf; /* @@@ use VPCI */
 	this->pvc.sap_addr.vpi = q_fetch(&in_dsc,QF_vpi);
 	this->pvc.sap_addr.vci = q_fetch(&in_dsc,QF_vci);
-printf("VPI/VCI: %d/%d\n",this->pvc.sap_addr.vpi,this->pvc.sap_addr.vci);
+	diag(COMPONENT,DIAG_DEBUG,"VPI/VCI: %d/%d",this->pvc.sap_addr.vpi,
+	  this->pvc.sap_addr.vci);
     }
     if (q_present(&in_dsc,QF_cgpn)) { /* should handle E.164 too */
 	char buffer[MAX_ATM_ADDR_LEN+1];
@@ -224,8 +226,14 @@ static void send_status(SOCKET *sock,unsigned char cause,...)
 
     q_create(&dsc,q_buffer,MAX_Q_MSG);
     q_assign(&dsc,QF_msg_type,QMSG_STATUS);
-    q_assign(&dsc,QF_call_ref,sock->call_ref);
-    q_assign(&dsc,QF_call_state,(int) sock->q2931_state);
+    if (sock) {
+	q_assign(&dsc,QF_call_ref,sock->call_ref);
+	q_assign(&dsc,QF_call_state,(int) sock->q2931_state);
+    }
+    else {
+	q_assign(&dsc,QF_call_ref,0);
+	q_assign(&dsc,QF_call_state,0); /* REST 0 - Null */
+    }
     q_assign(&dsc,QF_cause,cause);
     va_start(ap,cause);
     switch (cause) {
@@ -233,6 +241,15 @@ static void send_status(SOCKET *sock,unsigned char cause,...)
 	case ATM_CV_INCOMP_MSG:
 	    q_assign(&dsc,QF_bad_msg_type,va_arg(ap,unsigned char));
 	    break;
+	case ATM_CV_MAND_IE_MISSING:
+	case ATM_CV_INVALID_IE:
+	    {
+		unsigned char ie;
+
+		ie = va_arg(ap,unsigned char);
+		q_write(&dsc,QF_ie_id6,&ie,1);
+		break;
+	    }
 	default:
     }
     va_end(ap);
@@ -241,6 +258,20 @@ static void send_status(SOCKET *sock,unsigned char cause,...)
 }
 
 
+static void send_status_enq(SOCKET *sock)
+{
+    Q_DSC dsc;
+    int size;
+
+    q_create(&dsc,q_buffer,MAX_Q_MSG);
+    q_assign(&dsc,QF_msg_type,QMSG_STATUS_ENQ);
+    q_assign(&dsc,QF_call_ref,sock->call_ref);
+    size = q_close(&dsc);
+    to_signaling(q_buffer,size);
+    /* @@@ should start T322 */
+}
+
+
 static void send_connect_ack(SOCKET *sock)
 {
     Q_DSC dsc;
@@ -254,6 +285,25 @@ static void send_connect_ack(SOCKET *sock)
 }
 
 
+static void send_restart_ack(int vpi,int vci)
+{
+    Q_DSC dsc;
+    int size;
+
+    q_create(&dsc,q_buffer,MAX_Q_MSG);
+    q_assign(&dsc,QF_msg_type,QMSG_REST_ACK);
+    q_assign(&dsc,QF_call_ref,0);
+    if (!vpi && !vci) q_assign(&dsc,QF_rst_class,ATM_RST_ALL_VC);
+    else {
+	q_assign(&dsc,QF_rst_class,ATM_RST_ALL_VC);
+	q_assign(&dsc,QF_vpi,vpi);
+	q_assign(&dsc,QF_vci,vci);
+    }
+    size = q_close(&dsc);
+    to_signaling(q_buffer,size);
+}
+
+
 static void q2931_call(SOCKET *sock,unsigned char mid)
 {
     int error;
@@ -288,10 +338,11 @@ static void q2931_call(SOCKET *sock,unsigned char mid)
 	    /* check for 2nd CALL_PROC @@@ */
 	    STOP_TIMER(sock);
 	    if (q_present(&in_dsc,QG_conn_id)) {
-		sock->pvc.sap_addr.itf = 0; /* @@@ should use VPCI */
+		sock->pvc.sap_addr.itf = sig_itf; /* @@@ should use VPCI */
 		sock->pvc.sap_addr.vpi = q_fetch(&in_dsc,QF_vpi);
 		sock->pvc.sap_addr.vci = q_fetch(&in_dsc,QF_vci);
-printf("VPI/VCI: %d/%d\n",sock->pvc.sap_addr.vpi,sock->pvc.sap_addr.vci);
+		diag(COMPONENT,DIAG_DEBUG,"VPI/VCI: %d/%d",
+		  sock->pvc.sap_addr.vpi,sock->pvc.sap_addr.vci);
 	    }
 	    START_TIMER(sock,T310);
 	    sock->q2931_state = qs_out_proc;
@@ -304,10 +355,11 @@ printf("VPI/VCI: %d/%d\n",sock->pvc.sap_addr.vpi,sock->pvc.sap_addr.vci);
 	    if (sock->state != ss_connecting) break;
 	    STOP_TIMER(sock);
 	    if (q_present(&in_dsc,QG_conn_id)) {
-		sock->pvc.sap_addr.itf = 0; /* @@@ should use VPCI */
+		sock->pvc.sap_addr.itf = sig_itf; /* @@@ should use VPCI */
 		sock->pvc.sap_addr.vpi = q_fetch(&in_dsc,QF_vpi);
 		sock->pvc.sap_addr.vci = q_fetch(&in_dsc,QF_vci);
-printf("VPI/VCI: %d/%d\n",sock->pvc.sap_addr.vpi,sock->pvc.sap_addr.vci);
+		diag(COMPONENT,DIAG_DEBUG,"VPI/VCI: %d/%d",
+		  sock->pvc.sap_addr.vpi,sock->pvc.sap_addr.vci);
 	    }
 	    error = 0;
 	    if (!sock->pvc.sap_addr.vpi && !sock->pvc.sap_addr.vci)
@@ -344,6 +396,13 @@ printf("VPI/VCI: %d/%d\n",sock->pvc.sap_addr.vpi,sock->pvc.sap_addr.vci);
 	      sock->call_ref);
 	    return;
 	case QMSG_RELEASE: /* all states */
+	    {
+		unsigned char cause;
+
+		cause = q_fetch(&in_dsc,QF_cause);
+		diag(COMPONENT,DIAG_DEBUG,"Cause %d (%s)",cause,cause > 127 ?
+		  "invalid cause" : cause_text[cause]);
+	    }
 	    switch (sock->state) {
 		case ss_connecting:
 		    set_error(sock,-ECONNREFUSED);
@@ -379,36 +438,59 @@ printf("VPI/VCI: %d/%d\n",sock->pvc.sap_addr.vpi,sock->pvc.sap_addr.vci);
 		    break;
 	    }
 	    break;
+	case QMSG_RESTART:
+		set_error(sock,-ENETRESET);
+		/* fall through */
 	case QMSG_STATUS: /* fall through when clearing */
 	case QMSG_REL_COMP: /* basically any state (except LISTENING,
 				  ZOMBIE, REL_IND) */
-	    switch (sock->state) {
-		case ss_connecting:
-		    set_error(sock,-ECONNREFUSED);
-		    /* fall through */
-		case ss_accepting:
-		    set_error(sock,-ECONNRESET); /* ERESTARTSYS ? */
-		    /* fall through */
-		case ss_rel_req:
-		    send_close(sock);
-		    /* fall through */
-		case ss_wait_rel:
-		    STOP_TIMER(sock);
-		    free_sock(sock);
-		    return;
-		case ss_connected:
-		    diag(COMPONENT,DIAG_INFO,"Passive clsoe (CR 0x%06X)",
-		      sock->call_ref);
-		    send_close(sock);
-		    /* fall through */
-		case ss_hold:
-		    new_state(sock,ss_wait_close);
-		    return;
-		case ss_indicated:
-		    new_state(sock,ss_zombie);
-		    return;
-		default:
-		    break;
+	    {
+		unsigned char cause;
+
+		if (mid != QMSG_REL_COMP) cause = 0;
+		else {
+		    cause = q_fetch(&in_dsc,QF_cause);
+		    diag(COMPONENT,DIAG_DEBUG,"Cause %d (%s)",cause,
+		      cause > 127 ? "invalid cause" : cause_text[cause]);
+		}
+		switch (sock->state) {
+		    case ss_connecting:
+			set_error(sock,cause == ATM_CV_UNALLOC ?
+			  -EADDRNOTAVAIL : cause == ATM_CV_RES_UNAVAIL ||
+#ifdef UNI31
+			  cause == ATM_CV_UCR_UNAVAIL ||
+#endif
+			  cause == ATM_CV_NO_ROUTE_DEST ? -EHOSTUNREACH :
+			  cause == ATM_CV_NUM_CHANGED ? -EREMCHG :
+			  cause == ATM_CV_DEST_OOO ? -EHOSTDOWN :
+			  -ECONNREFUSED);
+			/* fall through */
+		    case ss_accepting:
+			set_error(sock,-ECONNRESET); /* ERESTARTSYS ? */
+			/* fall through */
+		    case ss_rel_req:
+			send_close(sock);
+			/* fall through */
+		    case ss_wait_rel:
+			STOP_TIMER(sock);
+			free_sock(sock);
+			return;
+		    case ss_connected:
+			diag(COMPONENT,DIAG_INFO,"Passive close (CR 0x%06X)",
+			  sock->call_ref);
+			sock->error = -sock->error; /* context is still around */
+			send_close(sock);
+			/* fall through */
+		    case ss_hold:
+			new_state(sock,ss_wait_close);
+			return;
+		    case ss_indicated:
+			new_state(sock,ss_zombie);
+			return;
+		    default:
+			break;
+		}
+		break; /* fail */
 	    }
 	default:
 	    diag(COMPONENT,DIAG_WARN,"Bad Q.2931 message %d",mid);
@@ -423,6 +505,49 @@ printf("VPI/VCI: %d/%d\n",sock->pvc.sap_addr.vpi,sock->pvc.sap_addr.vci);
 }
 
 
+static void clear_all_calls(void)
+{
+    SOCKET *curr,*next;
+
+    for (curr = sockets; curr; curr = next) {
+	next = curr->next;
+	if (curr->q2931_state != qs_null) q2931_call(curr,QMSG_RESTART);
+    }
+}
+
+
+void clear_all_calls_on_T309(void)
+{
+    clear_all_calls();
+    t309 = NULL;
+}
+
+
+void saal_failure(void)
+{
+    SOCKET *curr,*next;
+
+    for (curr = sockets; curr; curr = next) {
+	next = curr->next;
+	if (curr->q2931_state != qs_null)
+		if (curr->q2931_state != qs_active)
+			q2931_call(curr,QMSG_RESTART);
+		else if (!t309) t309 = start_timer(T309_TIME,on_T309,NULL);
+    }
+}
+
+
+void saal_okay(void)
+{
+    SOCKET *curr;
+
+    if (!t309) return;
+    stop_timer(t309);
+    t309 = NULL;
+    for (curr = sockets; curr; curr = curr->next)
+	if (curr->q2931_state != qs_null) send_status_enq(curr);
+}
+
 
 void to_q2931(void *msg,int size)
 {
@@ -432,12 +557,50 @@ void to_q2931(void *msg,int size)
 
     q_open(&in_dsc,msg,size);
     call_ref = q_fetch(&in_dsc,QF_call_ref)^0x800000;
+    mid = q_fetch(&in_dsc,QF_msg_type);
+    if (mid == QMSG_REST_ACK) return;
+    if (mid == QMSG_RESTART) { /* 5.5.5.2 */
+	int class;
+
+	class = q_fetch(&in_dsc,QF_rst_class);
+	switch (class) {
+	    case ATM_RST_IND_VC:
+		{
+		    int vpi,vci;
+
+		    if (!q_present(&in_dsc,QG_conn_id)) {
+			send_status(NULL,ATM_CV_MAND_IE_MISSING,
+			  ATM_IE_CONN_ID);
+			return;
+		    }
+		    vpi = q_fetch(&in_dsc,QF_vpi);
+		    vci = q_fetch(&in_dsc,QF_vci);
+		    for (curr = sockets; curr; curr = curr->next)
+			if (curr->pvc.sap_addr.vpi == vpi &&
+			  curr->pvc.sap_addr.vci == vci) break;
+		    if (!curr) {
+			send_status(NULL,ATM_CV_INVALID_IE,
+			  ATM_IE_CONN_ID);
+			return;
+		    }
+		    q2931_call(curr,mid);
+		    send_restart_ack(vpi,vci);
+		}
+		break;
+	    case ATM_RST_ALL_VC:
+		clear_all_calls();
+		send_restart_ack(0,0);
+		break;
+	    default:
+		send_status(NULL,ATM_CV_INVALID_IE,ATM_IE_RESTART);
+	}
+	return;
+    }
     if (!(call_ref & 0x7fffff)) {
 	return; /* bad things happen ... @@@ */
     }
     for (curr = sockets; curr; curr = curr->next)
 	if (curr->call_ref == call_ref) break;
-    mid = q_fetch(&in_dsc,QF_msg_type);
     diag(COMPONENT,DIAG_DEBUG,"FROM NET: %s (0x%02X) CR 0x%06lx for 0x%lx",
       mid2name(((unsigned char *) msg)[5]),((unsigned char *)msg)[5],call_ref,
       curr ? curr->id : 0);
