@@ -199,7 +199,7 @@ static void print_pdu(SSCOP_DSC *dsc,const char *label,unsigned char type,
 		len -= 2;
 	    }
 	    if (!len)
-		diag(COMPONENT,DIAG_DEBUG,"%s    <last is missing>)",label);
+		diag(COMPONENT,DIAG_DEBUG,"%s    <last is absent>)",label);
 	    else diag(COMPONENT,DIAG_DEBUG,"%s    <next is> %d)",label,
 		  ntohl(*list));
 	    break;
@@ -555,9 +555,9 @@ static void next_state(SSCOP_DSC *dsc,SSCOP_STATE state)
 
 static void bad_pdu(SSCOP_DSC *dsc,unsigned char type)
 {
-		  /*          111111 */
-		  /* 123456789012345 */
-    maa_error(dsc,"?1CxFJKDALGHIyzM"[type],0);
+		/*           111111 */
+		/* 0123456789012345 */
+    maa_error(dsc,"?BCxFJKDALGHIyzM"[type],0);
 }
 
 
