@@ -1,6 +1,6 @@
 /* timeout.c - Processing of Q.2931 timeout events */
  
-/* Written 1995 by Werner Almesberger, EPFL-LRC */
+/* Written 1995,1996 by Werner Almesberger, EPFL-LRC */
  
 
 #include <stdio.h>
@@ -66,6 +66,13 @@ void on_T308_2(void *user) /* WAIT_REL or REL_REQ */
 }
 
 
+void on_T309(void *user)
+{
+    diag(COMPONENT,DIAG_DEBUG,"T309 has expired");
+    clear_all_calls_on_T309();
+}
+
+
 void on_T310(void *user)
 {
     on_T303(user);
